/* Define any variables used later */

/* Serial Communication Variables */
// the selected peripheral device
var port = null;
// reader object
var reader = null;
// writer object
var writer = null;


/* HTML Elements  */
// status connection (text is updated with connection status)
const stat = document.querySelector("#status"); // HTML element w/ ID of "status"
// Slider for Rotation Speed
const slider = document.getElementById("bar"); // HTML element w/ ID of "bar"
// TIE-Fighter option (clickable box)
const tie = document.getElementById("tie-fighter"); // HTML element w/ ID of "tie-fighter"
// Circle option (clickable box)
const circle = document.getElementById("circle"); // HTML element w/ ID of "circle"
// Other Lissajous option (clickable box)
const custom = document.getElementById("custom-fig"); // HTML element w/ ID of "custom-fig"
// Disconnect button
const disconn = document.querySelector("#disconnect"); // HTML element w/ ID of "disconnect"
// Send Button
const send = document.getElementById("rotate");



/* Interacting with HTML: Event Listeners and oninput() slider function */

/* SENDING SHAPE */
tie.addEventListener("click", () => {	// if tie-fighter is clicked

  console.log("a");
  console.log("rotation: ", slider.value);
  textTx("a");

});

circle.addEventListener("click", () => {   // if circle is clicked

  console.log("b");
  console.log("rotation: ", slider.value);
  textTx("b");

});

custom.addEventListener("click", () => {   // if right lissajous figure is clicked

  console.log("c");
  console.log("rotation: ", slider.value);
  textTx("c");

});

/* SENDING ROTATION */

send.addEventListener("click", () => { // if 'Send' button is clicked

   console.log("rotation: ", slider.value);
   if (slider.value == 0) { textTx("d");console.log("rotation: ", slider.value);}
   else if (slider.value == 0.5) { textTx("e");console.log("rotation: ", slider.value);}
   else if (slider.value == 1) { textTx("f");console.log("rotation: ", slider.value);}
   else if (slider.value == 1.5) { textTx("g");console.log("rotation: ", slider.value);}

});


// Update HTML for rotation speed 
slider.oninput = async function () {

  //purely update the html text
  rotation.innerHTML = "Speed: " + this.value + " Hz";


/* option to update rotation realtime */
// uncomment code below to auto-update rotation speed as slider value changes
/*
   console.log("rotation: ", slider.value);
if (slider.value == 0) { textTx("d");console.log("rotation: ", slider.value);}
else if (slider.value == 0.5) { textTx("e");console.log("rotation: ", slider.value);}
else if (slider.value == 1) { textTx("f");console.log("rotation: ", slider.value);}
else if (slider.value == 1.5) { textTx("g");console.log("rotation: ", slider.value);}
*/

};


/* Disconnecting - User clicked the 'disconnect' button to close the serial port. */
disconn.addEventListener("click", async () => {	// When 'Disconnect' button clicked

  console.log("disconnecting...");
  disconnect();					// call disconnect() function
  stat.innerHTML = "STATUS: Disconnected"; // update status on screen
  console.log("Port disconnected!"); // update status in console

});


/* FUNCTIONS */

/* CONNECT() */
async function connect() {
  // Prompt user to select any serial port. [option to omit filters]
  // OMIT FILTERS: port = await navigator.serial.requestPort().catch((err) => {
  try {
    port = await navigator.serial.requestPort();
    const { usbVendorId, usbProductId } = port.getInfo();
  } catch (err) {
    console.log("Error on request");
    console.log(err);
  }
  //set baud rate to user's input. [option to prompt user for custom baud rate]
  //CUSTOM BAUD RATE: await port.open({ baudRate: prompt("Baud rate") });
  try {
    await port.open({ baudRate: 9600 });
    const { usbVendorId, usbProductId } = port.getInfo();
  } catch (err) {
    console.log("Error on open");
    console.log(err);
  }
  console.log("Port Connected!"); // update status in console
  console.log(port);
  stat.innerHTML = "STATUS: Connected"; // update status on HTML/screen

  textRx(); // constantly receive information on loop until disconnected

  if (port.writable) {
    writer = port.writable.getWriter();
    await writer.ready; // resolve any potential stream backpressure
  }
}

/* DISCONNECT() */
async function disconnect() {
  // Cancel reader, release writer, and close port
  if (reader) {
    await reader.cancel();
    reader = null;
  }
  if (writer) {
    await writer.releaseLock();
    writer = null;
  }
  if (port) {
    await port.close();
    port = null;
  }
}

/* TEXTTX() */
async function textTx(msg) {
  // msg is a string
  const encoder = new TextEncoder(); // encodes string as uint8's
  if (writer) {
    const encMessage = encoder.encode(msg); // encode message
    // resolve any potential stream backpressure
    await writer.ready.catch((err) => {
      console.error(`Ready error: ${err}`);
    });
    //transmit encoded message
    await writer.write(encMessage).catch((err) => {
      console.error(`Chunk error: ${err}`);
    });
    // resolve any potential stream backpressure
    await writer.ready;
  }
}

/* TEXTRX() */
async function textRx() {
  console.log("Read Loop Init");
  // resolve errors related to port = null
  if (!port) {
    return;
  }
  const decoder = new TextDecoder(); // decodes uint8's to a string

  //continuously read (loop)
  if (port.readable) {
    reader = port.readable.getReader();
    console.log("Read Loop Started");
    while (true) {
      const { value, done } = await reader.read();

      // decoder.decode(value) is any received data - handle here
      
      // done = true if reader is canceled; needs to release
      if (done) {
        reader.releaseLock();
        break;
      }
    }
  }
  console.log("Read Loop Stopped. Closing Serial Port.");
}


/* Event Listeners for serial port -- more info on API Docs */
// Automatically inform user a port has been detected. (ex. a USB is plugged in)
navigator.serial.addEventListener("connect", (event) => {
  alert("A port is available!");
  console.log(event);
});
// Automatically inform user a port has been disconnected. (ex. a USB is unplugged)
navigator.serial.addEventListener("disconnect", (event) => {
  alert("A port was disconnected!");
  console.log(event);
});

// Prints browser-API compatability to console
if ("serial" in navigator) {
  console.log("// The Web Serial API is supported");
} else {
  console.log("// The Web Serial API is not supported");
}
if ("serial" in navigator && "forget" in SerialPort.prototype) {
  console.log("// forget() is supported.");
} else {
  console.log("// forget() is not supported.");
}
