;=================================================================
; begin main program
;=================================================================
init:
	mov tmod, #20h 	; set timer 1 for auto reload - mode 2
	mov tcon, #41h 	; run counter 1 and set edge trig ints
	mov th1, #0fdh 	; set 9600 baud with xtal=11.059mhz
	mov scon, #50h 	; set serial control reg for 8 bit data
					; and mode 1
					
	mov P1, #00h	; initialize the LEDs to be all off (no shape, no rotation)

lazloop:
	clr ri 			; flush the serial input buffer
	lcall getcmd 	; read the single-letter command
	mov r2, a 		; put the command number in R2
	ljmp nway 		; branch to a monitor routine

endloop:	 		; come here after command has finished
	sjmp lazloop 	; loop forever in monitor loop


;=================================================================
; monitor jump table
;=================================================================	
jumtab:
	.dw badcmd ; command '@' 00
	.dw tfight ; command 'a' 01		tie fighter 
	.dw circle ; command 'b' 02		circle 
	.dw figure ; command 'c' 03		lissajous
	.dw slorot ; command 'd' 04 	slowest rotation 0 Hz
	.dw medrot ; command 'e' 05		medium rotation 0.5 Hz
	.dw hghrot ; command 'f' 06		high rotation 1 Hz
	.dw maxrot ; command 'g' 07 	fastest rotation 1.5 Hz
	.dw badcmd ; command 'h' 08
	.dw badcmd ; command 'i' 09
	.dw badcmd ; command 'j' 0a
	.dw badcmd ; command 'k' 0b
	.dw badcmd ; command 'l' 0c
	.dw badcmd ; command 'm' 0d
	.dw badcmd ; command 'n' 0e
	.dw badcmd ; command 'o' 0f
	.dw badcmd ; command 'p' 10
	.dw badcmd ; command 'q' 11
	.dw badcmd ; command 'r' 12 
	.dw badcmd ; command 's' 13
	.dw badcmd ; command 't' 14
	.dw badcmd ; command 'u' 15
	.dw badcmd ; command 'v' 16
	.dw badcmd ; command 'w' 17 
	.dw badcmd ; command 'x' 18
	.dw badcmd ; command 'y' 19
	.dw badcmd ; command 'z' 1a
	
;=================================================================
; begin subroutines
;=================================================================
	

;=================================================================
; subroutine tfight
; this subroutine is called when the R31JP receives 
; an "a" char in the serial receive buffer and drives
; lazerdillo to draw a tie fighter; it keeps the current 
; rotation setting 
;=================================================================	
tfight:
	mov A, P1		; save the current speed setting
	anl A, #0Fh		; mask off the top nibble
	add A, #0A0h	; add 10 (A in hex) to the top nibble 
	mov P1, A
	ljmp endloop 

;=================================================================
; subroutine circle
; this subroutine is called when the R31JP receives 
; an "b" char in the serial receive buffer and drives
; lazerdillo to draw a circle; it keeps the current 
; rotation setting 
;=================================================================	
circle:
	mov A, P1		; save the current speed setting
	anl A, #0Fh		; mask off the top nibble
	add A, #0B0h	; add 11 (B in hex) to the top nibble 
	mov P1, A
	ljmp endloop 

;=================================================================
; subroutine figure
; this subroutine is called when the R31JP receives 
; an "c" char in the serial receive buffer and drives
; lazerdillo to draw a lissajous figure with a = 1 and b = 4 
; it keeps the current rotation setting 
;=================================================================	
figure:
	mov A, P1		; save the current speed setting
	anl A, #0Fh		; mask off the top nibble
	add A, #0C0h	; add 12 (C in hex) to the top nibble 
	mov P1, A
	ljmp endloop 

;=================================================================
; subroutine slorot
; this subroutine is called when the R31JP receives 
; an "d" char in the serial receive buffer and makes the 
; currently drawn shape have no rotation
;=================================================================	
slorot:
	mov A, P1 		; save the current shape setting 
	anl A, #0F0h	; mask off the bottom nibble 
	mov P1, A
	ljmp endloop

;=================================================================
; subroutine medrot
; this subroutine is called when the R31JP receives 
; an "e" char in the serial receive buffer and makes the 
; currently drawn shape have a rotation of 0.5 Hz
;=================================================================	
medrot:
	mov A, P1 		; save the current shape setting 
	anl A, #0F0h	; mask off the bottom nibble 
	add A, #01h		; add 1 to the bottom nibble 
	mov P1, A
	ljmp endloop	

;=================================================================
; subroutine hghrot
; this subroutine is called when the R31JP receives 
; an "f" char in the serial receive buffer and makes the 
; currently drawn shape have a rotation of 1 Hz
;=================================================================	
hghrot:
	mov A, P1 		; save the current shape setting 
	anl A, #0F0h	; mask off the bottom nibble 
	add A, #02h		; add 2 to the bottom nibble 
	mov P1, A
	ljmp endloop

;=================================================================
; subroutine maxrot
; this subroutine is called when the R31JP receives 
; an "g" char in the serial receive buffer and makes the 
; currently drawn shape have a rotation of 1.5 Hz
;=================================================================	
maxrot:
	mov A, P1 		; save the current shape setting 
	anl A, #0F0h	; mask off the bottom nibble 
	add A, #04h		; add 4 to the bottom nibble 
	mov P1, A
	ljmp endloop

;===============================================================
; subroutine nway
; this routine branches (jumps) to the appropriate monitor
; routine. the routine number is in r2
;================================================================
nway:
	mov dptr, #jumtab 	; point dptr at beginning of jump table
	mov a, r2 			; load acc with monitor routine number
	rl a 				; multiply by two.
	inc a 				; load first vector onto stack
	movc a, @a+dptr 	; 		" 		"
	push acc 			; 		" 		"
	mov a, r2 			; load acc with monitor routine number
	rl a 				; multiply by two
	movc a, @a+dptr 	; load second vector onto stack
	push acc 			; 		" 		"
	ret 				; jump to start of monitor routine

;===============================================================
; subroutine getcmd
; this routine gets the command line. currently only a
; single-letter command is read - all command line parameters
; must be parsed by the individual routines.
;===============================================================
getcmd:
	lcall getchr 	; get the single-letter command
	clr acc.5 		; make upper case
	clr C 			; clear the carry flag
	subb a, #'@' 	; convert to command number
	jnc cmdok1 		; letter command must be above '@'
	lcall badpar
cmdok1:
	push acc 		; save command number
	subb a, #1Bh 	; command number must be 1Ah or less
	jc cmdok2
	lcall badpar 	; no need to pop acc since badpar
					; initializes the system

cmdok2:
	pop acc 		; recall command number
	ret
	
;*****************************************************************
; monitor support routines
;*****************************************************************
badpar:
	mov P1, #00h
	ljmp endloop
	
badcmd:
	mov P1, #00h
	ljmp endloop

;*****************************************************************
; general purpose routines
;*****************************************************************

;===============================================================
; subroutine getchr
; this routine reads in a chr from the serial port and saves it
; in the accumulator.
;===============================================================
getchr:
	jnb ri, getchr 	; wait till character received
	mov a, sbuf 	; get character
	anl a, #7fh 	; mask off 8th bit
	clr ri 			; clear serial status bit
	ret
